# Copyright 2015 OpenStack Foundation
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""
NSX-V3 Distributed Firewall
"""
from oslo_log import log

from vmware_nsx._i18n import _, _LW
from vmware_nsx.common import exceptions as nsx_exc
from vmware_nsx.common import utils
from vmware_nsx.nsxlib.v3 import client as nsxclient

LOG = log.getLogger(__name__)

# firewall section types
LAYER3 = 'LAYER3'

INSERT_BEFORE = 'insert_before'
INSERT_BOTTOM = 'insert_bottom'

# firewall rule actions
ALLOW = 'ALLOW'
DROP = 'DROP'
REJECT = 'REJECT'

# filtering operators and expressions
EQUALS = 'EQUALS'
NSGROUP_SIMPLE_EXPRESSION = 'NSGroupSimpleExpression'

# nsgroup members update actions
ADD_MEMBERS = 'ADD_MEMBERS'
REMOVE_MEMBERS = 'REMOVE_MEMBERS'

NSGROUP = 'NSGroup'
LOGICAL_SWITCH = 'LogicalSwitch'
LOGICAL_PORT = 'LogicalPort'
IPV4ADDRESS = 'IPv4Address'
IPV6ADDRESS = 'IPv6Address'

IN = 'IN'
OUT = 'OUT'
IN_OUT = 'IN_OUT'

# NSServices resource types
L4_PORT_SET_NSSERVICE = 'L4PortSetNSService'
ICMP_TYPE_NSSERVICE = 'ICMPTypeNSService'
IP_PROTOCOL_NSSERVICE = 'IPProtocolNSService'

TCP = 'TCP'
UDP = 'UDP'
ICMPV4 = 'ICMPv4'
ICMPV6 = 'ICMPv6'

IPV4 = 'IPV4'
IPV6 = 'IPV6'
IPV4_IPV6 = 'IPV4_IPV6'


class NSGroupMemberNotFound(nsx_exc.NsxPluginException):
    message = _("Could not find NSGroup %(nsgroup_id)s member %(member_id)s "
                "for removal.")


class NSGroupIsFull(nsx_exc.NsxPluginException):
    message = _("NSGroup %(nsgroup_id)s contains has reached its maximum "
                "capacity, unable to add additional members.")


def get_nsservice(resource_type, **properties):
    service = {'resource_type': resource_type}
    service.update(properties)
    return {'service': service}


def create_nsgroup(display_name, description, tags):
    body = {'display_name': display_name,
            'description': description,
            'tags': tags,
            'members': []}
    return nsxclient.create_resource('ns-groups', body)


def list_nsgroups():
    return nsxclient.get_resource(
        'ns-groups?populate_references=false').get('results', [])


@utils.retry_upon_exception_nsxv3(nsx_exc.StaleRevision)
def update_nsgroup(nsgroup_id, display_name, description):
    nsgroup = read_nsgroup(nsgroup_id)
    nsgroup.update({'display_name': display_name,
                    'description': description})
    return nsxclient.update_resource('ns-groups/%s' % nsgroup_id, nsgroup)


def get_nsgroup_member_expression(target_type, target_id):
    return {'resource_type': NSGROUP_SIMPLE_EXPRESSION,
            'target_property': 'id',
            'target_type': target_type,
            'op': EQUALS,
            'value': target_id}


@utils.retry_upon_exception_nsxv3(nsx_exc.ManagerError)
def _update_nsgroup_with_members(nsgroup_id, members, action):
    members_update = 'ns-groups/%s?action=%s' % (nsgroup_id, action)
    return nsxclient.create_resource(members_update, members)


def add_nsgroup_member(nsgroup_id, target_type, target_id):
    member_expr = get_nsgroup_member_expression(target_type, target_id)
    members = {'members': [member_expr]}
    try:
        return _update_nsgroup_with_members(nsgroup_id, members, ADD_MEMBERS)
    except nsx_exc.StaleRevision:
        raise
    except nsx_exc.ManagerError:
        # REVISIT(roeyc): A ManagerError might have been raised for a
        # different reason, e.g - NSGroup does not exists.
        LOG.warning(_LW("Failed to add %(target_type)s %(target_id)s to "
                        "NSGroup %(nsgroup_id)s"),
                    {'target_type': target_type,
                     'target_id': target_id,
                     'nsgroup_id': nsgroup_id})
        raise NSGroupIsFull(nsgroup_id=nsgroup_id)


def remove_nsgroup_member(nsgroup_id, target_type, target_id, verify=False):
    member_expr = get_nsgroup_member_expression(target_type, target_id)
    members = {'members': [member_expr]}
    try:
        return _update_nsgroup_with_members(
            nsgroup_id, members, REMOVE_MEMBERS)
    except nsx_exc.ManagerError:
        if verify:
            raise NSGroupMemberNotFound(member_id=target_id,
                                        nsgroup_id=nsgroup_id)


def read_nsgroup(nsgroup_id):
    return nsxclient.get_resource(
        'ns-groups/%s?populate_references=true' % nsgroup_id)


def delete_nsgroup(nsgroup_id):
    return nsxclient.delete_resource('ns-groups/%s?force=true' % nsgroup_id)


def _build_section(display_name, description, applied_tos, tags):
    return {'display_name': display_name,
            'description': description,
            'stateful': True,
            'section_type': LAYER3,
            'applied_tos': [get_nsgroup_reference(t_id)
                            for t_id in applied_tos],
            'tags': tags}


def create_empty_section(display_name, description, applied_tos, tags,
                         operation=INSERT_BOTTOM, other_section=None):
    resource = 'firewall/sections?operation=%s' % operation
    body = _build_section(display_name, description, applied_tos, tags)
    if other_section:
        resource += '&id=%s' % other_section
    return nsxclient.create_resource(resource, body)


@utils.retry_upon_exception_nsxv3(nsx_exc.StaleRevision)
def update_section(section_id, display_name, description, applied_tos=None):
    resource = 'firewall/sections/%s' % section_id
    section = read_section(section_id)
    section.update({'display_name': display_name,
                    'description': description})
    if applied_tos is not None:
        section['applied_tos'] = [get_nsgroup_reference(nsg_id)
                                  for nsg_id in applied_tos]
    return nsxclient.update_resource(resource, section)


def read_section(section_id):
    resource = 'firewall/sections/%s' % section_id
    return nsxclient.get_resource(resource)


def list_sections():
    resource = 'firewall/sections'
    return nsxclient.get_resource(resource).get('results', [])


def delete_section(section_id):
    resource = 'firewall/sections/%s?cascade=true' % section_id
    return nsxclient.delete_resource(resource)


def get_nsgroup_reference(nsgroup_id):
    return {'target_id': nsgroup_id,
            'target_type': NSGROUP}


def get_ip_cidr_reference(ip_cidr_block, ip_protocol):
    target_type = IPV4ADDRESS if ip_protocol == IPV4 else IPV6ADDRESS
    return {'target_id': ip_cidr_block,
            'target_type': target_type}


def get_firewall_rule_dict(display_name, source=None, destination=None,
                           direction=IN_OUT, ip_protocol=IPV4_IPV6,
                           service=None, action=ALLOW):
    return {'display_name': display_name,
            'sources': [source] if source else [],
            'destinations': [destination] if destination else [],
            'direction': direction,
            'ip_protocol': ip_protocol,
            'services': [service] if service else [],
            'action': action}


def add_rule_in_section(rule, section_id):
    resource = 'firewall/sections/%s/rules' % section_id
    params = '?operation=insert_bottom'
    return nsxclient.create_resource(resource + params, rule)


def add_rules_in_section(rules, section_id):
    resource = 'firewall/sections/%s/rules' % section_id
    params = '?action=create_multiple&operation=insert_bottom'
    return nsxclient.create_resource(resource + params, {'rules': rules})


def delete_rule(section_id, rule_id):
    resource = 'firewall/sections/%s/rules/%s' % (section_id, rule_id)
    return nsxclient.delete_resource(resource)



# Copyright 2015 VMware Inc
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import time

from tempest.lib.common.utils import data_utils

from tempest.api.network import base_routers as base
from tempest import config
from tempest import test
from vmware_nsx_tempest.services import nsxv_client

CONF = config.CONF
ROUTER_SIZE = ('compact', 'large', 'xlarge', 'quadlarge')


class ExcRouterTest(base.BaseRouterTest):
    """
    Test class for exclusive router type, which is 1:1 mapping of
    NSX-v service edge. Tests will sipped if the router-type
    extension is not enabled.
    """

    @classmethod
    def skip_checks(cls):
        super(ExcRouterTest, cls).skip_checks()
        if not test.is_extension_enabled('nsxv-router-type', 'network'):
            msg = "router-type extension is not enabled"
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(ExcRouterTest, cls).setup_clients()

    @classmethod
    def resource_setup(cls):
        super(ExcRouterTest, cls).resource_setup()
        cls.tenant_cidr = (CONF.network.tenant_network_cidr
                           if cls._ip_version == 4 else
                           CONF.network.tenant_network_v6_cidr)
        manager_ip = re.search(r"(\d{1,3}\.){3}\d{1,3}",
                               CONF.nsxv.manager_uri).group(0)
        cls.vsm = nsxv_client.VSMClient(
            manager_ip, CONF.nsxv.user, CONF.nsxv.password)

    @test.attr(type='nsxv')
    @test.idempotent_id('ac1639a0-2a8d-4c68-bccd-54849fd45f86')
    def test_create_exc_router(self):
        """
        Test create an exclusive router. After creation, check nsx_v
        backend create service for the exclusive router.
        """
        name = data_utils.rand_name('router-')
        router = self.client.create_router(
            name, external_gateway_info={
                "network_id": CONF.network.public_network_id},
            admin_state_up=False, router_type='exclusive')
        self.addCleanup(self._delete_router, router['router']['id'])
        router_nsxv_name = '%s-%s' % (router['router']['name'],
                                      router['router']['id'])
        self.assertEqual(router['router']['name'], name)
        exc_edge = self.vsm.get_edge(router_nsxv_name)
        self.assertTrue(exc_edge is not None)
        self.assertEqual(exc_edge['edgeType'], 'gatewayServices')

    @test.attr(type='nsxv')
    @test.idempotent_id('c4b94988-0bc7-11e5-9203-0050568833db')
    def test_update_exc_router(self):
        """
        Test update an exclusive router
        """
        name = data_utils.rand_name('router-')
        router = self.client.create_router(
            name, external_gateway_info={
                "network_id": CONF.network.public_network_id},
            admin_state_up=False, router_type='exclusive')
        self.addCleanup(self._delete_router, router['router']['id'])
        self.assertEqual(router['router']['name'], name)
        updated_name = 'updated' + name
        update_body = self.client.update_router(router['router']['id'],
                                                name=updated_name)
        self.assertEqual(update_body['router']['name'], updated_name)

    @test.attr(type='nsxv')
    @test.idempotent_id('a0ff5afa-0bcc-11e5-9203-0050568833db')
    def test_list_show_exc_router(self):
        """
        Test list and show exclusive router.
        """
        name = data_utils.rand_name('router-')
        router = self.client.create_router(
            name, external_gateway_info={
                "network_id": CONF.network.public_network_id},
            admin_state_up=False, router_type='exclusive')
        self.addCleanup(self._delete_router, router['router']['id'])
        self.assertEqual(router['router']['name'], name)
        # Show details of exclusive router
        show_body = self.client.show_router(router['router']['id'])
        self.assertEqual(show_body['router']['name'], name)
        self.assertEqual(show_body['router']['admin_state_up'], False)
        # List routers and verify if created router in list
        list_body = self.client.list_routers()
        routers_list = [r['id'] for r in list_body['routers']]
        self.assertIn(router['router']['id'], routers_list)

    @test.attr(type='nsxv')
    @test.idempotent_id('adef8d1e-0bce-11e5-9203-0050568833db')
    def test_delete_exc_router(self):
        """
        Test create, update, and delete an exclusive router
        """
        name = data_utils.rand_name('router-')
        router = self.client.create_router(
            name, external_gateway_info={
                "network_id": CONF.network.public_network_id},
            admin_state_up=False, router_type='exclusive')
        self.assertEqual(router['router']['name'], name)
        # Update the name of the exclusive router
        updated_name = 'updated' + name
        update_body = self.client.update_router(router['router']['id'],
                                                name=updated_name)
        self.assertEqual(update_body['router']['name'], updated_name)
        # Delete the exclusive router and verify it has been deleted
        # from nsxv backend
        self.client.delete_router(router['router']['id'])
        list_body = self.client.list_routers()
        routers_list = [r['id'] for r in list_body['routers']]
        self.assertNotIn(router['router']['id'], routers_list)
        nsxv_edge_name = "%s-%s" % (name, router['router']['id'])
        self.assertEqual(self.vsm.get_edge(nsxv_edge_name), None)

    @test.attr(type='nsxv')
    @test.idempotent_id('d75fbcd5-c8cb-49ea-a868-ada12fd8c87f')
    def test_create_update_delete_compact_router(self):
        self.do_create_update_delete_router_with_size('compact')

    @test.attr(type='nsxv')
    @test.idempotent_id('da00c74f-81e6-4ef9-8aca-8e0345b376e9')
    def test_create_update_delete_large_router(self):
        self.do_create_update_delete_router_with_size('large', 20.0)

    @test.attr(type='nsxv')
    @test.idempotent_id('091dad07-6044-4ca3-b16c-54a3ef92254b')
    def test_create_update_delete_xlarge_router(self):
        self.do_create_update_delete_router_with_size('xlarge', 20.0)

    @test.attr(type='nsxv')
    @test.idempotent_id('0f69bf8a-4b06-47ac-a3f7-eedba95fd395')
    def test_create_update_delete_quadlarge_router(self):
        self.do_create_update_delete_router_with_size('quadlarge', 30.0)

    def do_create_update_delete_router_with_size(self,
                                                 router_size,
                                                 del_waitfor=10.0,
                                                 del_interval=1.5):
        name = data_utils.rand_name('rtr-%s' % router_size)
        router = self.client.create_router(
            name, external_gateway_info={
                "network_id": CONF.network.public_network_id},
            admin_state_up=False, router_type='exclusive',
            router_size=router_size)
        self.assertEqual(router['router']['name'], name)
        # Update the name of the exclusive router
        updated_name = 'updated' + name
        update_body = self.client.update_router(router['router']['id'],
                                                name=updated_name)
        self.assertEqual(update_body['router']['name'], updated_name)
        # Delete the exclusive router and verify it has been deleted
        # from nsxv backend
        self.client.delete_router(router['router']['id'])
        list_body = self.client.list_routers()
        routers_list = [r['id'] for r in list_body['routers']]
        self.assertNotIn(router['router']['id'], routers_list)
        nsxv_edge_name = "%s-%s" % (name, router['router']['id'])
        wait_till = time.time() + del_waitfor
        while (time.time() < wait_till):
            try:
                self.assertEqual(self.vsm.get_edge(nsxv_edge_name), None)
                return
            except Exception:
                time.sleep(del_interval)
        # last try. Fail if nesx_edge still exists
        fail_msg = ("%s router nsxv_edge[%s] still exists after %s seconds." %
                    (router_size, nsxv_edge_name, del_waitfor))
        self.assertEqual(self.vsm.get_edge(nsxv_edge_name), None, fail_msg)



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import test

from oslo_log import log as logging
from tempest.lib.common.utils import data_utils
import test_subnets as SNET

LOG = logging.getLogger(__name__)


class FlatNetworksTestJSON(SNET.SubnetTestJSON):
    _interface = 'json'
    _provider_network_body = {
        'name': data_utils.rand_name('FLAT-network'),
        'provider:network_type': 'flat'}

    @classmethod
    def resource_setup(cls):
        super(FlatNetworksTestJSON, cls).resource_setup()

    def _create_network(self, _auto_clean_up=True, network_name=None,
                        **kwargs):
        network_name = network_name or data_utils.rand_name('flat-netwk')
        # self.create_network expect network_name
        # self.admin_client.create_network()
        #   and self.client.create_network() expect name
        post_body = {'name': network_name,
                     'provider:network_type': 'flat'}
        post_body.update(kwargs)
        LOG.debug("create FLAT network: %s", str(post_body))
        body = self.admin_networks_client.create_network(**post_body)
        network = body['network']
        if _auto_clean_up:
            self.addCleanup(self._try_delete_network, network['id'])
        return network

    @test.idempotent_id('dc2f2f46-0577-4e2a-b35d-3c8c8bbce5bf')
    def test_create_network(self):
        # Create a network as an admin user specifying the
        # flat network type attribute
        network = self._create_network()
        # Verifies router:network_type parameter
        self.assertIsNotNone(network['id'])
        self.assertEqual(network.get('provider:network_type'), 'flat')

    @test.idempotent_id('777fc335-b26c-42ea-9759-c71dff2ce1c6')
    def test_update_network(self):
        # Update flat network as an admin user specifying the
        # flat network attribute
        network = self._create_network(shared=True, _auto_clean_up=False)
        self.assertEqual(network.get('shared'), True)
        new_name = network['name'] + "-updated"
        update_body = {'shared': False, 'name': new_name}
        body = self.update_network(network['id'], **update_body)
        updated_network = body['network']
        # Verify that name and shared parameters were updated
        self.assertEqual(updated_network['shared'], False)
        self.assertEqual(updated_network['name'], new_name)
        # get flat network attributes and verify them
        body = self.show_network(network['id'])
        updated_network = body['network']
        # Verify that name and shared parameters were updated
        self.assertEqual(updated_network['shared'], False)
        self.assertEqual(updated_network['name'], new_name)
        self.assertEqual(updated_network['status'], network['status'])
        self.assertEqual(updated_network['subnets'], network['subnets'])
        self._delete_network(network['id'])

    @test.idempotent_id('1dfc1c11-e838-464c-85b2-ed5e4c477c64')
    def test_list_networks(self):
        # Create flat network
        network = self._create_network(shared=True)
        # List networks as a normal user and confirm it is available
        body = self.list_networks(client=self.networks_client)
        network_list = [net['id'] for net in body['networks']]
        self.assertIn(network['id'], network_list)
        update_body = {'shared': False}
        body = self.update_network(network['id'], **update_body)
        # List networks as a normal user and confirm it is not available
        body = self.list_networks(client=self.networks_client)
        network_list = [net['id'] for net in body['networks']]
        self.assertNotIn(network['id'], network_list)

    @test.idempotent_id('b5649fe2-a214-4105-8053-1825a877c45b')
    def test_show_network_attributes(self):
        # Create flat network
        network = self._create_network(shared=True)
        # Show a flat network as a normal user and confirm the
        # flat network attribute is returned.
        body = self.show_network(network['id'], client=self.networks_client)
        show_net = body['network']
        self.assertEqual(network['name'], show_net['name'])
        self.assertEqual(network['id'], show_net['id'])
        # provider attributes are for admin only
        body = self.show_network(network['id'])
        show_net = body['network']
        net_attr_list = show_net.keys()
        for attr in ('admin_state_up', 'port_security_enabled', 'shared',
                     'status', 'subnets', 'tenant_id', 'router:external',
                     'provider:network_type', 'provider:physical_network',
                     'provider:segmentation_id'):
            self.assertIn(attr, net_attr_list)



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base_provider as base
from tempest.common import custom_matchers
from tempest import config
from tempest import test

import netaddr
from oslo_log import log as logging
import six
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF

LOG = logging.getLogger(__name__)


class SubnetTestJSON(base.BaseAdminNetworkTest):
    _provider_network_body = {}

    """
    [NOTE: This module copied/modified from api/network/test_networks.py
        to create provider networks/subnets tests]

    Tests the following operations in the Neutron API using the REST client for
    Neutron:

        create a network for a tenant
        list tenant's networks
        show a tenant network details
        create a subnet for a tenant
        list tenant's subnets
        show a tenant subnet details
        network update
        subnet update
        delete a network also deletes its subnets

        All subnet tests are run once with ipv4 and once with ipv6.

    v2.0 of the Neutron API is assumed. It is also assumed that the following
    options are defined in the [network] section of etc/tempest.conf:

        tenant_network_cidr with a block of cidr's from which smaller blocks
        can be allocated for tenant ipv4 subnets

        tenant_network_v6_cidr is the equivalent for ipv6 subnets

        tenant_network_mask_bits with the mask bits to be used to partition the
        block defined by tenant_network_cidr

        tenant_network_v6_mask_bits is the equivalent for ipv6 subnets
    """

    @classmethod
    def resource_setup(cls):
        super(SubnetTestJSON, cls).resource_setup()
        for k, v in cls._provider_network_body.items():
            if not v:
                cls._provider_network_body.pop(k)
        body = cls.create_network(client=cls.admin_networks_client,
                                  **cls._provider_network_body)
        cls.network = body['network']
        cls.name = cls.network['name']
        cls.subnet = cls._create_subnet_with_last_subnet_block(cls.network)
        cls.cidr = cls.subnet['cidr']
        cls._subnet_data = {6: {'gateway':
                                str(cls._get_gateway_from_tempest_conf(6)),
                                'allocation_pools':
                                cls._get_allocation_pools_from_gateway(6),
                                'dns_nameservers': ['2001:4860:4860::8844',
                                                    '2001:4860:4860::8888'],
                                'host_routes': [{'destination': '2001::/64',
                                                 'nexthop': '2003::1'}],
                                'new_host_routes': [{'destination':
                                                     '2001::/64',
                                                     'nexthop': '2005::1'}],
                                'new_dns_nameservers':
                                ['2001:4860:4860::7744',
                                 '2001:4860:4860::7888']},
                            4: {'gateway':
                                str(cls._get_gateway_from_tempest_conf(4)),
                                'allocation_pools':
                                cls._get_allocation_pools_from_gateway(4),
                                'dns_nameservers': ['8.8.4.4', '8.8.8.8'],
                                'host_routes': [{'destination': '10.20.0.0/32',
                                                 'nexthop': '10.100.1.1'}],
                                'new_host_routes': [{'destination':
                                                     '10.20.0.0/32',
                                                     'nexthop':
                                                     '10.100.1.2'}],
                                'new_dns_nameservers': ['7.8.8.8', '7.8.4.4']}}

    @classmethod
    def _create_subnet_with_last_subnet_block(cls, network, ip_version=4):
        """Derive last subnet CIDR block from tenant CIDR and
           create the subnet with that derived CIDR
        """
        if ip_version == 4:
            cidr = netaddr.IPNetwork(CONF.network.tenant_network_cidr)
            mask_bits = CONF.network.tenant_network_mask_bits
        elif ip_version == 6:
            cidr = netaddr.IPNetwork(CONF.network.tenant_network_v6_cidr)
            mask_bits = CONF.network.tenant_network_v6_mask_bits

        subnet_cidr = list(cidr.subnet(mask_bits))[-1]
        gateway_ip = str(netaddr.IPAddress(subnet_cidr) + 1)
        body = cls.create_subnet(network, gateway=gateway_ip,
                                 cidr=subnet_cidr, mask_bits=mask_bits)
        return body['subnet']

    @classmethod
    def _get_gateway_from_tempest_conf(cls, ip_version):
        """Return first subnet gateway for configured CIDR."""
        if ip_version == 4:
            cidr = netaddr.IPNetwork(CONF.network.tenant_network_cidr)
            mask_bits = CONF.network.tenant_network_mask_bits
        elif ip_version == 6:
            cidr = netaddr.IPNetwork(CONF.network.tenant_network_v6_cidr)
            mask_bits = CONF.network.tenant_network_v6_mask_bits

        if mask_bits >= cidr.prefixlen:
            return netaddr.IPAddress(cidr) + 1
        else:
            for subnet in cidr.subnet(mask_bits):
                return netaddr.IPAddress(subnet) + 1

    @classmethod
    def _get_allocation_pools_from_gateway(cls, ip_version):
        """Return allocation range for subnet of given gateway."""
        gateway = cls._get_gateway_from_tempest_conf(ip_version)
        return [{'start': str(gateway + 2), 'end': str(gateway + 3)}]

    def subnet_dict(self, include_keys):
        """Return a subnet dict which has include_keys and their corresponding
           value from self._subnet_data
        """
        return dict((key, self._subnet_data[self._ip_version][key])
                    for key in include_keys)

    def _create_network(self, _auto_clean_up=True, network_name=None,
                        **kwargs):
        network_name = network_name or data_utils.rand_name('adm-netwk')
        post_body = {'name': network_name}
        post_body.update(kwargs)
        LOG.debug("create ADM network: %s", str(post_body))
        body = self.create_network(client=self.admin_networks_client,
                                   **post_body)
        network = body['network']
        if _auto_clean_up:
            self.addCleanup(self._try_delete_network, network['id'])
        return network

    # when you call _delete_network() you mean it is part of test,
    # so we will not pass exception
    def _delete_network(self, net_id):
        self._remove_network_from_book(net_id)
        return self.delete_network(net_id)

    def _remove_network_from_book(self, net_id):
        for idx, netwk_info in zip(range(0, len(self.admin_netwk_info)),
                                   self.admin_netwk_info):
            net_client, network = netwk_info
            if network['id'] == net_id:
                self.admin_netwk_info.pop(idx)
                return

    # call _try_delete_network() for teardown purpose, so pass exception
    def _try_delete_network(self, net_id):
        # delete network, if it exists
        self._remove_network_from_book(net_id)
        try:
            self.delete_network(net_id)
        # if network is not found, this means it was deleted in the test
        except exceptions.NotFound:
            pass

    # by default, subnet will be deleted when its network is deleted
    def _create_subnet(self, network, gateway='', cidr=None, mask_bits=None,
                       ip_version=None, cidr_offset=0,
                       _auto_clean_up=False, **kwargs):
        body = self.create_subnet(network,
                                  gateway=gateway,
                                  cidr=cidr,
                                  mask_bits=mask_bits,
                                  ip_version=ip_version,
                                  cidr_offset=cidr_offset,
                                  **kwargs)
        subnet = body['subnet']
        if _auto_clean_up:
            self.addCleanup(self._try_delete_subnet, subnet['id'])
        return subnet

    def _try_delete_subnet(self, net_id):
        # delete subnet, if it exists
        try:
            self.delete_subnet(net_id)
        # if network is not found, this means it was deleted in the test
        except exceptions.NotFound:
            pass

    def _compare_resource_attrs(self, actual, expected):
        exclude_keys = set(actual).symmetric_difference(expected)
        self.assertThat(actual, custom_matchers.MatchesDictExceptForKeys(
                        expected, exclude_keys))

    def _create_verify_delete_subnet(self, cidr=None, mask_bits=None,
                                     **kwargs):
        network = self._create_network(_auto_clean_up=True)
        net_id = network['id']
        gateway = kwargs.pop('gateway', None)
        subnet = self._create_subnet(network, gateway, cidr, mask_bits,
                                     **kwargs)
        compare_args_full = dict(gateway_ip=gateway, cidr=cidr,
                                 mask_bits=mask_bits, **kwargs)
        compare_args = (dict((k, v)
                        for k, v in six.iteritems(compare_args_full)
                        if v is not None))

        if 'dns_nameservers' in set(subnet).intersection(compare_args):
            self.assertEqual(sorted(compare_args['dns_nameservers']),
                             sorted(subnet['dns_nameservers']))
            del subnet['dns_nameservers'], compare_args['dns_nameservers']

        self._compare_resource_attrs(subnet, compare_args)
        self._delete_network(net_id)

    @test.idempotent_id('2ecbc3ab-93dd-44bf-a827-95beeb008e9a')
    def test_create_update_delete_network_subnet(self):
        # Create a network
        network = self._create_network(_auto_clean_up=True)
        net_id = network['id']
        self.assertEqual('ACTIVE', network['status'])
        # Verify network update
        new_name = data_utils.rand_name('new-adm-netwk')
        body = self.update_network(net_id, name=new_name)
        updated_net = body['network']
        self.assertEqual(updated_net['name'], new_name)
        # Find a cidr that is not in use yet and create a subnet with it
        subnet = self._create_subnet(network)
        subnet_id = subnet['id']
        # Verify subnet update
        new_name = data_utils.rand_name('new-subnet')
        body = self.update_subnet(subnet_id, name=new_name)
        updated_subnet = body['subnet']
        self.assertEqual(updated_subnet['name'], new_name)
        self._delete_network(net_id)

    @test.idempotent_id('a2cf6398-aece-4256-88a6-0dfe8aa44975')
    def test_show_network(self):
        # Verify the details of a network
        body = self.show_network(self.network['id'])
        network = body['network']
        for key in ['id', 'name']:
            self.assertEqual(network[key], self.network[key])

    @test.idempotent_id('5b42067d-4b9d-4f04-bb6a-adb9756ebe0c')
    def test_show_network_fields(self):
        # Verify specific fields of a network
        fields = ['id', 'name']
        body = self.show_network(self.network['id'], fields=fields)
        network = body['network']
        self.assertEqual(sorted(network.keys()), sorted(fields))
        for field_name in fields:
            self.assertEqual(network[field_name], self.network[field_name])

    @test.idempotent_id('324be3c2-457d-4e21-b0b3-5106bbbf1a28')
    def test_list_networks(self):
        # Verify the network exists in the list of all networks
        body = self.list_networks()
        networks = [network['id'] for network in body['networks']
                    if network['id'] == self.network['id']]
        self.assertNotEmpty(networks, "Created network not found in the list")

    @test.idempotent_id('3a934a8d-6b52-427e-af49-3dfdd224fdeb')
    def test_list_networks_fields(self):
        # Verify specific fields of the networks
        fields = ['id', 'name']
        body = self.list_networks(fields=fields)
        networks = body['networks']
        self.assertNotEmpty(networks, "Network list returned is empty")
        for network in networks:
            self.assertEqual(sorted(network.keys()), sorted(fields))

    @test.idempotent_id('5f6616c4-bfa7-4308-8eab-f45d75c94c6d')
    def test_show_subnet(self):
        # Verify the details of a subnet
        body = self.show_subnet(self.subnet['id'])
        subnet = body['subnet']
        self.assertNotEmpty(subnet, "Subnet returned has no fields")
        for key in ['id', 'cidr']:
            self.assertIn(key, subnet)
            self.assertEqual(subnet[key], self.subnet[key])

    @test.idempotent_id('2f326955-551e-4e9e-a4f6-e5db77c34c8d')
    def test_show_subnet_fields(self):
        # Verify specific fields of a subnet
        fields = ['id', 'network_id']
        body = self.show_subnet(self.subnet['id'], fields=fields)
        subnet = body['subnet']
        self.assertEqual(sorted(subnet.keys()), sorted(fields))
        for field_name in fields:
            self.assertEqual(subnet[field_name], self.subnet[field_name])

    @test.idempotent_id('66631557-2466-4827-bba6-d961b0242be3')
    def test_list_subnets(self):
        # Verify the subnet exists in the list of all subnets
        body = self.list_subnets()
        subnets = [subnet['id'] for subnet in body['subnets']
                   if subnet['id'] == self.subnet['id']]
        self.assertNotEmpty(subnets, "Created subnet not found in the list")

    @test.idempotent_id('3d5ea69b-f122-43e7-b7f4-c78586629eb8')
    def test_list_subnets_fields(self):
        # Verify specific fields of subnets
        fields = ['id', 'network_id']
        body = self.list_subnets(fields=fields)
        subnets = body['subnets']
        self.assertNotEmpty(subnets, "Subnet list returned is empty")
        for subnet in subnets:
            self.assertEqual(sorted(subnet.keys()), sorted(fields))

    @test.idempotent_id('e966bb2f-402c-49b7-8147-b275cee584c4')
    def test_delete_network_with_subnet(self):
        # Creates a network
        network = self._create_network(_auto_clean_up=True)
        net_id = network['id']

        # Find a cidr that is not in use yet and create a subnet with it
        subnet = self._create_subnet(network)
        subnet_id = subnet['id']

        # Delete network while the subnet still exists
        self._delete_network(net_id)

        # Verify that the subnet got automatically deleted.
        self.assertRaises(exceptions.NotFound,
                          self.show_subnet, subnet_id)

    @test.idempotent_id('8aba0e1b-4b70-4181-a8a4-792c08db699d')
    def test_create_delete_subnet_without_gateway(self):
        self._create_verify_delete_subnet()

    @test.idempotent_id('67364a4b-6725-4dbe-84cf-504bdb20ac06')
    def test_create_delete_subnet_with_gw(self):
        self._create_verify_delete_subnet(
            **self.subnet_dict(['gateway']))

    @test.idempotent_id('f8f43e65-5090-4902-b5d2-2b610505cca6')
    def test_create_delete_subnet_with_allocation_pools(self):
        self._create_verify_delete_subnet(
            **self.subnet_dict(['allocation_pools']))

    @test.idempotent_id('5b085669-97e6-48e0-b99e-315a9b4d8482')
    def test_create_delete_subnet_with_gw_and_allocation_pools(self):
        self._create_verify_delete_subnet(**self.subnet_dict(
            ['gateway', 'allocation_pools']))

    @decorators.skip_because(bug="1501827")
    def test_create_delete_subnet_with_host_routes_and_dns_nameservers(self):
        self._create_verify_delete_subnet(
            **self.subnet_dict(['host_routes', 'dns_nameservers']))

    @test.idempotent_id('df518c87-b817-48b5-9365-bd1daaf68955')
    def test_create_delete_subnet_with_dns_nameservers(self):
        self._create_verify_delete_subnet(
            **self.subnet_dict(['dns_nameservers']))

    @test.idempotent_id('b6822feb-6760-4052-b550-f0fe8bac7451')
    def test_create_delete_subnet_with_dhcp_enabled(self):
        self._create_verify_delete_subnet(enable_dhcp=True)

    @decorators.skip_because(bug="1501827")
    def test_update_subnet_gw_dns_host_routes_dhcp(self):
        network = self._create_network(_auto_clean_up=True)
        subnet_attrs = ['gateway', 'host_routes',
                        'dns_nameservers', 'allocation_pools']
        subnet_dict = self.subnet_dict(subnet_attrs)
        subnet = self._create_subnet(network, **subnet_dict)
        subnet_id = subnet['id']
        new_gateway = str(netaddr.IPAddress(
                          self._subnet_data[self._ip_version]['gateway']) + 1)
        # Verify subnet update
        new_host_routes = self._subnet_data[self._ip_version][
            'new_host_routes']

        new_dns_nameservers = self._subnet_data[self._ip_version][
            'new_dns_nameservers']
        kwargs = {'host_routes': new_host_routes,
                  'dns_nameservers': new_dns_nameservers,
                  'gateway_ip': new_gateway, 'enable_dhcp': True}

        new_name = "New_subnet"
        body = self.update_subnet(subnet_id, name=new_name, **kwargs)
        updated_subnet = body['subnet']
        kwargs['name'] = new_name
        self.assertEqual(sorted(updated_subnet['dns_nameservers']),
                         sorted(kwargs['dns_nameservers']))
        del subnet['dns_nameservers'], kwargs['dns_nameservers']

        self._compare_resource_attrs(updated_subnet, kwargs)
        self._delete_network(network['id'])

    @test.idempotent_id('a5caa7d9-ab71-4278-a57c-d6631b7474f8')
    def test_update_subnet_gw_dns_dhcp(self):
        network = self._create_network(_auto_clean_up=True)
        subnet_attrs = ['gateway',
                        'dns_nameservers', 'allocation_pools']
        subnet_dict = self.subnet_dict(subnet_attrs)
        subnet = self._create_subnet(network, **subnet_dict)
        subnet_id = subnet['id']
        new_gateway = str(netaddr.IPAddress(
                          self._subnet_data[self._ip_version]['gateway']) + 1)
        # Verify subnet update
        new_dns_nameservers = self._subnet_data[self._ip_version][
            'new_dns_nameservers']
        kwargs = {'dns_nameservers': new_dns_nameservers,
                  'gateway_ip': new_gateway, 'enable_dhcp': True}

        new_name = "New_subnet"
        body = self.update_subnet(subnet_id, name=new_name, **kwargs)
        updated_subnet = body['subnet']
        kwargs['name'] = new_name
        self.assertEqual(sorted(updated_subnet['dns_nameservers']),
                         sorted(kwargs['dns_nameservers']))
        del subnet['dns_nameservers'], kwargs['dns_nameservers']

        self._compare_resource_attrs(updated_subnet, kwargs)
        self._delete_network(network['id'])

    @decorators.skip_because(bug="1501827")
    def test_create_delete_subnet_all_attributes(self):
        self._create_verify_delete_subnet(
            enable_dhcp=True,
            **self.subnet_dict(['gateway',
                                'host_routes',
                                'dns_nameservers']))

    @test.idempotent_id('969f20b2-7eb5-44f5-98cd-381545b7c7e7')
    def test_create_delete_subnet_with_gw_dns(self):
        self._create_verify_delete_subnet(
            enable_dhcp=True,
            **self.subnet_dict(['gateway',
                                'dns_nameservers']))

    @test.idempotent_id('3c4c36a1-684b-4e89-8e71-d528f19324a0')
    def test_add_upd_del_multiple_overlapping_networks_subnet(self):
        r0, R1 = 0, 3   # (todo) get from CONF
        return self._add_upd_del_multiple_networks_subnet(
            r0, R1, "ovla-netwk")

    @test.idempotent_id('5267bf9d-de82-4af9-914a-8320e9f4c38c')
    def test_add_upd_del_multiple_nonoverlapping_networks_subnet(self):
        r0, R1 = 1, 4   # (todo) get from CONF
        return self._add_upd_del_multiple_networks_subnet(
            r0, R1, "noov-netwk", _step_cidr=2)

    def _add_upd_del_multiple_networks_subnet(self, r0, R1,
                                              name_prefix="m-network",
                                              _step_cidr=0):
        m_name = data_utils.rand_name(name_prefix)
        netwk = []
        for x in range(r0, R1):
            network = self._create_network(_auto_clean_up=True)
            net_id = network['id']
            self.assertEqual('ACTIVE', network['status'])
            new_name = m_name + "-%02d" % x
            body = self.update_network(net_id, name=new_name)
            network = body['network']
            cidr_offset = (x * _step_cidr) if _step_cidr > 0 else 0
            subnet = self._create_subnet(network, cidr_offset=cidr_offset)
            subnet_id = subnet['id']
            netwk.append([x, net_id, subnet_id])
        for x, net_id, subnet_id in netwk:
            # make sure subnet is updatable after creation
            new_name = m_name + "-%02d-snet" % x
            body = self.update_subnet(subnet_id, name=new_name)
            updated_subnet = body['subnet']
            self.assertEqual(updated_subnet['name'], new_name)
            self._delete_network(net_id)



# Copyright 2012 OpenStack Foundation
# Copyright 2015 VMware Inc
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import collections
import re
import time

from oslo_log import log as logging
from tempest.lib.common.utils import data_utils
import testtools

from tempest import config
from tempest import exceptions
from tempest.scenario import manager
from tempest.services.network import resources as net_resources
from tempest import test

CONF = config.CONF
FIP_OPS_TIMEOUT = 10
LOG = logging.getLogger(__name__)

Floating_IP_tuple = collections.namedtuple('Floating_IP_tuple',
                                           ['floating_ip', 'server'])


class TestDvrBasicOps(manager.NetworkScenarioTest):

    """
    This smoke test suite assumes that Nova has been configured to
    boot VM's with Neutron-managed networking, and attempts to
    verify network connectivity as follows:

     There are presumed to be two types of networks: tenant and
     public.  A tenant network may or may not be reachable from the
     Tempest host.  A public network is assumed to be reachable from
     the Tempest host, and it should be possible to associate a public
     ('floating') IP address with a tenant ('fixed') IP address to
     facilitate external connectivity to a potentially unroutable
     tenant IP address.

     This test suite can be configured to test network connectivity to
     a VM via a tenant network, a public network, or both.  If both
     networking types are to be evaluated, tests that need to be
     executed remotely on the VM (via ssh) will only be run against
     one of the networks (to minimize test execution time).

     Determine which types of networks to test as follows:

     * Configure tenant network checks (via the
       'tenant_networks_reachable' key) if the Tempest host should
       have direct connectivity to tenant networks.  This is likely to
       be the case if Tempest is running on the same host as a
       single-node devstack installation with IP namespaces disabled.

     * Configure checks for a public network if a public network has
       been configured prior to the test suite being run and if the
       Tempest host should have connectivity to that public network.
       Checking connectivity for a public network requires that a
       value be provided for 'public_network_id'.  A value can
       optionally be provided for 'public_router_id' if tenants will
       use a shared router to access a public network (as is likely to
       be the case when IP namespaces are not enabled).  If a value is
       not provided for 'public_router_id', a router will be created
       for each tenant and use the network identified by
       'public_network_id' as its gateway.

    """

    @classmethod
    def skip_checks(cls):
        super(TestDvrBasicOps, cls).skip_checks()
        if not (CONF.network.tenant_networks_reachable
                or CONF.network.public_network_id):
            msg = ('Either tenant_networks_reachable must be "true", or '
                   'public_network_id must be defined.')
            raise cls.skipException(msg)
        for ext in ['router', 'security-group', 'dvr']:
            if not test.is_extension_enabled(ext, 'network'):
                msg = "%s extension not enabled." % ext
                raise cls.skipException(msg)

    @classmethod
    def setup_credentials(cls):
        # Ask framework to not create network resources for these tests.
        cls.set_network_resources()
        super(TestDvrBasicOps, cls).setup_credentials()

    def setUp(self):
        super(TestDvrBasicOps, self).setUp()
        self.keypairs = {}
        self.servers = []

    def _setup_network_and_servers(self, **kwargs):
        boot_with_port = kwargs.pop('boot_with_port', False)
        self.security_group = \
            self._create_security_group(tenant_id=self.tenant_id)
        self.network, self.subnet, self.router = self.create_networks(**kwargs)
        self.check_networks()

        self.port_id = None
        if boot_with_port:
            # create a port on the network and boot with that
            self.port_id = self._create_port(self.network['id']).id

        name = data_utils.rand_name('server-smoke')
        server = self._create_server(name, self.network, self.port_id)
        self._check_tenant_network_connectivity()

        floating_ip = self.create_floating_ip(server)
        self.floating_ip_tuple = Floating_IP_tuple(floating_ip, server)

    # overwrite super class who does not accept router attributes
    def create_networks(self, dns_nameservers=None, **kwargs):
        client = self.network_client
        networks_client = self.networks_client
        subnets_client = self.subnets_client
        network = self._create_network(
            client=client, networks_client=networks_client,
            tenant_id=self.tenant_id)

        router_kwargs = {}
        for k in kwargs.keys():
            if k in ('distributed', 'router_type', 'router_size'):
                router_kwargs[k] = kwargs.pop(k)
        router = self._create_router(**router_kwargs)
        router.set_gateway(CONF.network.public_network_id)

        subnet_kwargs = dict(network=network, client=client,
                             subnets_client=subnets_client)
        # use explicit check because empty list is a valid option
        if dns_nameservers is not None:
            subnet_kwargs['dns_nameservers'] = dns_nameservers
        subnet = self._create_subnet(**subnet_kwargs)
        subnet.add_to_router(router.id)
        return network, subnet, router

    # overwrite super class
    def _create_router(self, client=None, tenant_id=None,
                       namestart='router-smoke', **kwargs):
        if not client:
            client = self.network_client
        if not tenant_id:
            tenant_id = client.tenant_id
        name = data_utils.rand_name(namestart)
        result = client.create_router(name=name,
                                      admin_state_up=True,
                                      tenant_id=tenant_id,
                                      **kwargs)
        router = net_resources.DeletableRouter(client=client,
                                               **result['router'])
        self.assertEqual(router.name, name)
        self.addCleanup(self.delete_wrapper, router.delete)
        return router

    def check_networks(self):
        """
        Checks that we see the newly created network/subnet/router via
        checking the result of list_[networks,routers,subnets]
        """

        seen_nets = self._list_networks()
        seen_names = [n['name'] for n in seen_nets]
        seen_ids = [n['id'] for n in seen_nets]
        self.assertIn(self.network.name, seen_names)
        self.assertIn(self.network.id, seen_ids)

        if self.subnet:
            seen_subnets = self._list_subnets()
            seen_net_ids = [n['network_id'] for n in seen_subnets]
            seen_subnet_ids = [n['id'] for n in seen_subnets]
            self.assertIn(self.network.id, seen_net_ids)
            self.assertIn(self.subnet.id, seen_subnet_ids)

        if self.router:
            seen_routers = self._list_routers()
            seen_router_ids = [n['id'] for n in seen_routers]
            seen_router_names = [n['name'] for n in seen_routers]
            self.assertIn(self.router.name,
                          seen_router_names)
            self.assertIn(self.router.id,
                          seen_router_ids)

    def _create_server(self, name, network, port_id=None):
        keypair = self.create_keypair()
        self.keypairs[keypair['name']] = keypair
        security_groups = [{'name': self.security_group['name']}]
        create_kwargs = {
            'networks': [
                {'uuid': network.id},
            ],
            'key_name': keypair['name'],
            'security_groups': security_groups,
            'wait_until': 'ACTIVE',
        }
        if port_id is not None:
            create_kwargs['networks'][0]['port'] = port_id
        server = self.create_server(name=name, **create_kwargs)
        self.servers.append(server)
        return server

    def _get_server_key(self, server):
        return self.keypairs[server['key_name']]['private_key']

    def _check_tenant_network_connectivity(self):
        ssh_login = CONF.validation.image_ssh_user
        for server in self.servers:
            # call the common method in the parent class
            super(TestDvrBasicOps, self).\
                _check_tenant_network_connectivity(
                    server, ssh_login, self._get_server_key(server),
                    servers_for_debug=self.servers)

    def check_public_network_connectivity(
            self, should_connect=True, msg=None,
            should_check_floating_ip_status=True):
        """Verifies connectivty to a VM via public network and floating IP,
        and verifies floating IP has resource status is correct.

        :param should_connect: bool. determines if connectivity check is
        negative or positive.
        :param msg: Failure message to add to Error message. Should describe
        the place in the test scenario where the method was called,
        to indicate the context of the failure
        :param should_check_floating_ip_status: bool. should status of
        floating_ip be checked or not
        """
        ssh_login = CONF.validation.image_ssh_user
        floating_ip, server = self.floating_ip_tuple
        ip_address = floating_ip.floating_ip_address
        private_key = None
        floatingip_status = 'DOWN'
        if should_connect:
            private_key = self._get_server_key(server)
            floatingip_status = 'ACTIVE'
        # Check FloatingIP Status before initiating a connection
        if should_check_floating_ip_status:
            self.check_floating_ip_status(floating_ip, floatingip_status)
        # call the common method in the parent class
        super(TestDvrBasicOps, self).check_public_network_connectivity(
            ip_address, ssh_login, private_key, should_connect, msg,
            self.servers)

    def _disassociate_floating_ips(self):
        floating_ip, server = self.floating_ip_tuple
        self._disassociate_floating_ip(floating_ip)
        self.floating_ip_tuple = Floating_IP_tuple(
            floating_ip, None)

    def _reassociate_floating_ips(self):
        floating_ip, server = self.floating_ip_tuple
        name = data_utils.rand_name('new_server-smoke')
        # create a new server for the floating ip
        server = self._create_server(name, self.network)
        self._associate_floating_ip(floating_ip, server)
        self.floating_ip_tuple = Floating_IP_tuple(
            floating_ip, server)

    def _create_new_network(self, create_gateway=False):
        self.new_net = self._create_network(tenant_id=self.tenant_id)
        if create_gateway:
            self.new_subnet = self._create_subnet(
                network=self.new_net)
        else:
            self.new_subnet = self._create_subnet(
                network=self.new_net,
                gateway_ip=None)

    def _hotplug_server(self):
        old_floating_ip, server = self.floating_ip_tuple
        ip_address = old_floating_ip.floating_ip_address
        private_key = self._get_server_key(server)
        ssh_client = self.get_remote_client(ip_address,
                                            private_key=private_key)
        old_nic_list = self._get_server_nics(ssh_client)
        # get a port from a list of one item
        port_list = self._list_ports(device_id=server['id'])
        self.assertEqual(1, len(port_list))
        old_port = port_list[0]
        interface = self.interface_client.create_interface(
            server=server['id'],
            network_id=self.new_net.id)
        self.addCleanup(self.network_client.wait_for_resource_deletion,
                        'port',
                        interface['port_id'])
        self.addCleanup(self.delete_wrapper,
                        self.interface_client.delete_interface,
                        server['id'], interface['port_id'])

        def check_ports():
            self.new_port_list = [port for port in
                                  self._list_ports(device_id=server['id'])
                                  if port['id'] != old_port['id']]
            return len(self.new_port_list) == 1

        if not test.call_until_true(check_ports, CONF.network.build_timeout,
                                    CONF.network.build_interval):
            raise exceptions.TimeoutException(
                "No new port attached to the server in time (%s sec)! "
                "Old port: %s. Number of new ports: %d" % (
                    CONF.network.build_timeout, old_port,
                    len(self.new_port_list)))
        new_port = net_resources.DeletablePort(client=self.network_client,
                                               **self.new_port_list[0])

        def check_new_nic():
            new_nic_list = self._get_server_nics(ssh_client)
            self.diff_list = [n for n in new_nic_list if n not in old_nic_list]
            return len(self.diff_list) == 1

        if not test.call_until_true(check_new_nic, CONF.network.build_timeout,
                                    CONF.network.build_interval):
            raise exceptions.TimeoutException("Interface not visible on the "
                                              "guest after %s sec"
                                              % CONF.network.build_timeout)

        num, new_nic = self.diff_list[0]
        ssh_client.assign_static_ip(nic=new_nic,
                                    addr=new_port.fixed_ips[0]['ip_address'])
        ssh_client.turn_nic_on(nic=new_nic)

    def _get_server_nics(self, ssh_client):
        reg = re.compile(r'(?P<num>\d+): (?P<nic_name>\w+):')
        ipatxt = ssh_client.get_ip_list()
        return reg.findall(ipatxt)

    def _check_network_internal_connectivity(self, network,
                                             should_connect=True):
        """
        via ssh check VM internal connectivity:
        - ping internal gateway and DHCP port, implying in-tenant connectivity
        pinging both, because L3 and DHCP agents might be on different nodes
        """
        floating_ip, server = self.floating_ip_tuple
        # get internal ports' ips:
        # get all network ports in the new network
        internal_ips = (p['fixed_ips'][0]['ip_address'] for p in
                        self._list_ports(tenant_id=server['tenant_id'],
                                         network_id=network.id)
                        if (p['device_owner'].startswith('network') and
                            not p['device_owner'].endswith('dhcp')))

        self._check_server_connectivity(floating_ip,
                                        internal_ips,
                                        should_connect)

    def _check_network_external_connectivity(self):
        """
        ping public network default gateway to imply external connectivity

        """
        if not CONF.network.public_network_id:
            msg = 'public network not defined.'
            LOG.debug(msg)
            return

        # We ping the external IP from the instance using its floating IP
        # which is always IPv4, so we must only test connectivity to
        # external IPv4 IPs if the external network is dualstack.
        v4_subnets = [s for s in self._list_subnets(
            network_id=CONF.network.public_network_id) if s['ip_version'] == 4]
        self.assertEqual(1, len(v4_subnets),
                         "Found %d IPv4 subnets" % len(v4_subnets))

        external_ips = [v4_subnets[0]['gateway_ip']]
        self._check_server_connectivity(self.floating_ip_tuple.floating_ip,
                                        external_ips)

    def _check_server_connectivity(self, floating_ip, address_list,
                                   should_connect=True):
        ip_address = floating_ip.floating_ip_address
        private_key = self._get_server_key(self.floating_ip_tuple.server)
        ssh_source = self._ssh_to_server(ip_address, private_key)

        for remote_ip in address_list:
            if should_connect:
                msg = "Timed out waiting for "
                "%s to become reachable" % remote_ip
            else:
                msg = "ip address %s is reachable" % remote_ip
            try:
                self.assertTrue(self._check_remote_connectivity
                                (ssh_source, remote_ip, should_connect),
                                msg)
            except Exception:
                LOG.debug("Unable to access {dest} via ssh to "
                          "floating-ip {src}".format(dest=remote_ip,
                                                     src=floating_ip))
                raise

    @test.idempotent_id('62eb50a8-45f3-4eec-acc4-f01cee10a011')
    @test.services('compute', 'network')
    def test_dvr_network_basic_ops(self):
        """
        For a freshly-booted VM with an IP address ("port") on a given
            network:

        - the Tempest host can ping the IP address.  This implies, but
         does not guarantee (see the ssh check that follows), that the
         VM has been assigned the correct IP address and has
         connectivity to the Tempest host.

        - the Tempest host can perform key-based authentication to an
         ssh server hosted at the IP address.  This check guarantees
         that the IP address is associated with the target VM.

        - the Tempest host can ssh into the VM via the IP address and
         successfully execute the following:

         - ping an external IP address, implying external connectivity.

         - ping an external hostname, implying that dns is correctly
           configured.

         - ping an internal IP address, implying connectivity to another
           VM on the same network.

        - detach the floating-ip from the VM and verify that it becomes
        unreachable

        - associate detached floating ip to a new VM and verify connectivity.
        VMs are created with unique keypair so connectivity also asserts that
        floating IP is associated with the new VM instead of the old one

        Verifies that floating IP status is updated correctly after each change


        """
        self._setup_network_and_servers(distributed=True)
        LOG.debug("Sleeping %ss after associate floating ip %s" %
                  (FIP_OPS_TIMEOUT, self.floating_ip_tuple))
        self.check_public_network_connectivity(should_connect=True)
        self._check_network_internal_connectivity(network=self.network)
        self._check_network_external_connectivity()
        self._disassociate_floating_ips()
        LOG.debug("Sleeping %ss after disassociate floating ip %s" %
                  (FIP_OPS_TIMEOUT, self.floating_ip_tuple))
        self.check_public_network_connectivity(should_connect=False,
                                               msg="after disassociate "
                                                   "floating ip")
        self._reassociate_floating_ips()
        LOG.debug("Sleeping %ss after reassociate floating ip %s" %
                  (FIP_OPS_TIMEOUT, self.floating_ip_tuple))
        self.check_public_network_connectivity(should_connect=True,
                                               msg="after re-associate "
                                                   "floating ip")

    @test.idempotent_id('d99b62ec-28ce-44db-a195-edb74037a354')
    @testtools.skipIf(CONF.baremetal.driver_enabled,
                      'Baremetal relies on a shared physical network.')
    @test.services('compute', 'network')
    def test_dvr_connectivity_between_vms_on_different_networks(self):
        """
        For a freshly-booted VM with an IP address ("port") on a given
            network:

        - the Tempest host can ping the IP address.

        - the Tempest host can ssh into the VM via the IP address and
         successfully execute the following:

         - ping an external IP address, implying external connectivity.

         - ping an external hostname, implying that dns is correctly
           configured.

         - ping an internal IP address, implying connectivity to another
           VM on the same network.

        - Create another network on the same tenant with subnet, create
        an VM on the new network.

         - Ping the new VM from previous VM failed since the new network
         was not attached to router yet.

         - Attach the new network to the router, Ping the new VM from
         previous VM succeed.

        """
        self._setup_network_and_servers(distributed=True)
        LOG.debug("Sleeping %ss after associate floating ip %s" %
                  (FIP_OPS_TIMEOUT, self.floating_ip_tuple))
        time.sleep(FIP_OPS_TIMEOUT)
        self.check_public_network_connectivity(should_connect=True)
        self._check_network_internal_connectivity(network=self.network)
        self._check_network_external_connectivity()
        self._create_new_network(create_gateway=True)
        name = data_utils.rand_name('server-smoke')
        self._create_server(name, self.new_net)
        self._check_network_internal_connectivity(network=self.new_net,
                                                  should_connect=False)
        self.new_subnet.add_to_router(self.router.id)
        self._check_network_internal_connectivity(network=self.new_net,
                                                  should_connect=True)

    @test.idempotent_id('a73fd605-d55e-4151-b25e-41e7a7ff2258')
    @testtools.skipIf(CONF.baremetal.driver_enabled,
                      'Router state cannot be altered on a shared baremetal '
                      'network')
    @test.services('compute', 'network')
    def test_dvr_update_router_admin_state(self):
        """
        1. Check public connectivity before updating
                admin_state_up attribute of router to False
        2. Check public connectivity after updating
                admin_state_up attribute of router to False
        3. Check public connectivity after updating
                admin_state_up attribute of router to True
        """
        self._setup_network_and_servers(distributed=True)
        LOG.debug("Sleeping %ss after associate floating ip %s" %
                  (FIP_OPS_TIMEOUT, self.floating_ip_tuple))
        time.sleep(FIP_OPS_TIMEOUT)
        self.check_public_network_connectivity(
            should_connect=True, msg="before updating "
            "admin_state_up of router to False")
        self._update_router_admin_state(self.router, False)
        # TODO(alokmaurya): Remove should_check_floating_ip_status=False check
        # once bug 1396310 is fixed

        self.check_public_network_connectivity(
            should_connect=False, msg="after updating "
            "admin_state_up of router to False",
            should_check_floating_ip_status=False)
        self._update_router_admin_state(self.router, True)
        self.check_public_network_connectivity(
            should_connect=True, msg="after updating "
            "admin_state_up of router to True")



# Copyright 2015 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
NSX-V3 Plugin router module
"""
import copy

from neutron_lib import exceptions as n_exc
from oslo_log import log

from vmware_nsx._i18n import _, _LW
from vmware_nsx.common import exceptions as nsx_exc
from vmware_nsx.common import nsx_constants
from vmware_nsx.common import utils
from vmware_nsx.nsxlib import v3 as nsxlib

LOG = log.getLogger(__name__)

MIN_EDGE_NODE_NUM = 1

TIER0_ROUTER_LINK_PORT_NAME = "TIER0-RouterLinkPort"
TIER1_ROUTER_LINK_PORT_NAME = "TIER1-RouterLinkPort"
ROUTER_INTF_PORT_NAME = "Tier1-RouterDownLinkPort"

FIP_NAT_PRI = 900
GW_NAT_PRI = 1000


class RouterLib(object):

    def __init__(self, router_client, router_port_client):
        self._router_client = router_client
        self._router_port_client = router_port_client

    def validate_tier0(self, tier0_groups_dict, tier0_uuid):
        err_msg = None
        try:
            lrouter = self._router_client.get(tier0_uuid)
        except nsx_exc.ResourceNotFound:
            err_msg = (_("Tier0 router %s not found at the backend. Either a "
                         "valid UUID must be specified or a default tier0 "
                         "router UUID must be configured in nsx.ini") %
                       tier0_uuid)
        else:
            edge_cluster_uuid = lrouter.get('edge_cluster_id')
            if not edge_cluster_uuid:
                err_msg = _("Failed to get edge cluster uuid from tier0 "
                            "router %s at the backend") % lrouter
            else:
                edge_cluster = nsxlib.get_edge_cluster(edge_cluster_uuid)
                member_index_list = [member['member_index']
                                     for member in edge_cluster['members']]
                if len(member_index_list) < MIN_EDGE_NODE_NUM:
                    err_msg = _("%(act_num)s edge members found in "
                                "edge_cluster %(cluster_id)s, however we "
                                "require at least %(exp_num)s edge nodes "
                                "in edge cluster for use.") % {
                        'act_num': len(member_index_list),
                        'exp_num': MIN_EDGE_NODE_NUM,
                        'cluster_id': edge_cluster_uuid}
        if err_msg:
            raise n_exc.InvalidInput(error_message=err_msg)
        else:
            tier0_groups_dict[tier0_uuid] = {
                'edge_cluster_uuid': edge_cluster_uuid,
                'member_index_list': member_index_list}

    def add_router_link_port(self, tier1_uuid, tier0_uuid, edge_members,
                             tags):
        # Create Tier0 logical router link port
        t0_tags = copy.copy(tags)
        t0_tags = utils.add_v3_tag(t0_tags, 'os-tier0-uuid', tier0_uuid)
        tier0_link_port = self._router_port_client.create(
            tier0_uuid, display_name=TIER0_ROUTER_LINK_PORT_NAME, tags=t0_tags,
            resource_type=nsx_constants.LROUTERPORT_LINKONTIER0,
            logical_port_id=None,
            address_groups=None)
        linked_logical_port_id = tier0_link_port['id']
        # Create Tier1 logical router link port
        t1_tags = copy.copy(tags)
        t1_tags = utils.add_v3_tag(t1_tags, 'os-tier1-uuid', tier1_uuid)
        self._router_port_client.create(
            tier1_uuid, display_name=TIER1_ROUTER_LINK_PORT_NAME, tags=t1_tags,
            resource_type=nsx_constants.LROUTERPORT_LINKONTIER1,
            logical_port_id=linked_logical_port_id,
            address_groups=None)

    def remove_router_link_port(self, tier1_uuid, tier0_uuid):
        try:
            tier1_link_port = (
                self._router_port_client.get_tier1_link_port(tier1_uuid))
        except nsx_exc.ResourceNotFound:
            LOG.warning(_LW("Logical router link port for tier1 router: %s "
                            "not found at the backend"), tier1_uuid)
            return
        tier1_link_port_id = tier1_link_port['id']
        tier0_link_port_id = (
            tier1_link_port['linked_logical_router_port_id'].get('target_id'))
        self._router_port_client.delete(tier1_link_port_id)
        self._router_port_client.delete(tier0_link_port_id)

    def update_advertisement(self, logical_router_id, advertise_route_nat,
                             advertise_route_connected, enabled=True):
        return nsxlib.update_logical_router_advertisement(
            logical_router_id,
            advertise_nat_routes=advertise_route_nat,
            advertise_nsx_connected_routes=advertise_route_connected,
            enabled=enabled)

    def delete_gw_snat_rule(self, logical_router_id, gw_ip):
        return nsxlib.delete_nat_rule_by_values(logical_router_id,
                                                translated_network=gw_ip)

    def add_gw_snat_rule(self, logical_router_id, gw_ip):
        return nsxlib.add_nat_rule(logical_router_id, action="SNAT",
                                   translated_network=gw_ip,
                                   rule_priority=GW_NAT_PRI)

    def update_router_edge_cluster(self, nsx_router_id, edge_cluster_uuid):
        return self._router_client.update(nsx_router_id,
                                          edge_cluster_id=edge_cluster_uuid)

    def create_logical_router_intf_port_by_ls_id(self, logical_router_id,
                                                 display_name,
                                                 tags,
                                                 ls_id,
                                                 logical_switch_port_id,
                                                 address_groups):
        try:
            port = self._router_port_client.get_by_lswitch_id(ls_id)
        except nsx_exc.ResourceNotFound:
            return self._router_port_client.create(
                logical_router_id,
                display_name,
                tags,
                nsx_constants.LROUTERPORT_DOWNLINK,
                logical_switch_port_id,
                address_groups)
        else:
            return self._router_port_client.update(
                port['id'], subnets=address_groups)

    def add_fip_nat_rules(self, logical_router_id, ext_ip, int_ip):
        nsxlib.add_nat_rule(logical_router_id, action="SNAT",
                            translated_network=ext_ip,
                            source_net=int_ip,
                            rule_priority=FIP_NAT_PRI)
        nsxlib.add_nat_rule(logical_router_id, action="DNAT",
                            translated_network=int_ip,
                            dest_net=ext_ip,
                            rule_priority=FIP_NAT_PRI)

    def delete_fip_nat_rules(self, logical_router_id, ext_ip, int_ip):
        nsxlib.delete_nat_rule_by_values(logical_router_id,
                                         action="SNAT",
                                         translated_network=ext_ip,
                                         match_source_network=int_ip)
        nsxlib.delete_nat_rule_by_values(logical_router_id,
                                         action="DNAT",
                                         translated_network=int_ip,
                                         match_destination_network=ext_ip)

    def add_static_routes(self, nsx_router_id, route):
        return nsxlib.add_static_route(nsx_router_id, route['destination'],
                                       route['nexthop'])

    def delete_static_routes(self, nsx_router_id, route):
        return nsxlib.delete_static_route_by_values(
            nsx_router_id, dest_cidr=route['destination'],
            nexthop=route['nexthop'])



# Copyright 2015 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# This module inherents from resources module and enhances router functions
# and block subnet's add_to/delete_from_router so it is more similar to CLI.

from tempest.services.network import resources as n_resources


DELETABLE_CLASS_DEF = """class %(cls_name)s(n_resources.%(cls_name)s):
    pass
"""
IGNORE_LIST = ['DeletableSubnet', 'DeletableRouter']


# inhere Deletable<Class> from parent module
for cls_name in [x for x in dir(n_resources)
                 if x.startswith('Deletable') and x not in IGNORE_LIST]:
    class_def = DELETABLE_CLASS_DEF % dict(cls_name=cls_name)
    exec class_def


# Add/mod methods so we can use it while sustain original functions.
MSG_BLOCK_BY_ADMIN = "Block %s as router might be owned by ADMIN. " \
                     "Use DeletableRouter instead."


class DeletableSubnet(n_resources.DeletableSubnet):

    def __init__(self, *args, **kwargs):
        super(DeletableSubnet, self).__init__(*args, **kwargs)

    def add_to_router(self, router_id):
        raise Exception(MSG_BLOCK_BY_ADMIN % "add_to_router()")

    def delete_from_router(self, router_id):
        raise Exception(MSG_BLOCK_BY_ADMIN % "delete_from_router()")


# DeletableSubnet should not deal with router which when owned by ADMIN
# will raise privilege issue. Always let the router deals with interfaces.
class DeletableRouter(n_resources.DeletableRouter):
    def __init__(self, *args, **kwargs):
        super(DeletableRouter, self).__init__(*args, **kwargs)
        self._subnets = set()

    def set_gateway(self, network_id):
        return self.client.update_router(
            self.id,
            external_gateway_info=dict(network_id=network_id))

    def unset_gateway(self):
        return self.client.update_router(
            self.id,
            external_gateway_info=dict())

    def add_subnet(self, subnet):
        return self.add_interface(subnet)

    def add_interface(self, subnet):
        # should not let subnet add interface to router as
        # the router might be crated by admin.
        """
        self.client.add_router_interface_with_subnbet_id(
            self.id, subnet_id=subnet.id)
        """
        x_method(self.client, 'add_router_interface_with_subnet_id',
                 self.id, subnet_id=subnet.id)
        self._subnets.add(subnet)

    def delete_subnet(self, subnet):
        return self.delete_interface(subnet)

    def delete_interface(self, subnet):
        """
        self.client.remove_router_interface_with_subnet_id(
            self.id, subnet_id=subnet.id)
        """
        x_method(self.client, 'remove_router_interface_with_subnet_id',
                 self.id, subnet_id=subnet.id)
        try:
            self._subnets.remove(subnet)
        except Exception:
            pass

    def update_extra_routes(self, nexthop, destination):
        return self.client.update_extra_routes(self.id, nexthop, destination)

    # to-be-fixed by https://bugs.launchpad.net/tempest/+bug/1468600
    def update_extra_routes_future(self, routes):
        return self.client.update_extra_routes(self.id, routes)

    def delete_extra_routes(self):
        return self.client.delete_extra_routes(self.id)

    def delete(self):
        try:
            self.delete_extra_routes()
        except Exception:
            pass
        self.unset_gateway()
        for subnet in self._subnets.copy():
            self.delete_interface(subnet)
        super(DeletableRouter, self).delete()


# Workaround solution
def x_method(target_obj, method_name, *args, **kwargs):
    _method = getattr(target_obj, method_name, None)
    if _method is None:
        raise Exception("Method[%s] is not defined at instance[%s]" %
                        method_name, str(target_obj))
    results = _method(*args, **kwargs)
    return results



# Copyright 2015 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import mock
import six.moves.urllib.parse as urlparse

from oslo_config import cfg
from oslo_serialization import jsonutils
from requests import exceptions as requests_exceptions
from vmware_nsx.common import exceptions as nsx_exc
from vmware_nsx.nsxlib.v3 import client
from vmware_nsx.nsxlib.v3 import cluster
from vmware_nsx.tests.unit.nsx_v3 import mocks
from vmware_nsx.tests.unit.nsxlib.v3 import nsxlib_testcase


def _validate_conn_up(*args, **kwargs):
    return


def _validate_conn_down(*args, **kwargs):
    raise requests_exceptions.ConnectionError()


class RequestsHTTPProviderTestCase(nsxlib_testcase.NsxClientTestCase):

    def test_new_connection(self):
        mock_api = mock.Mock()
        mock_api.username = 'nsxuser'
        mock_api.password = 'nsxpassword'
        mock_api.retries = 100
        mock_api.insecure = True
        mock_api.ca_file = None
        mock_api.http_timeout = 99
        mock_api.conn_idle_timeout = 39
        provider = cluster.NSXRequestsHTTPProvider()
        session = provider.new_connection(
            mock_api, cluster.Provider('9.8.7.6', 'https://9.8.7.6'))

        self.assertEqual(session.auth, ('nsxuser', 'nsxpassword'))
        self.assertEqual(session.verify, False)
        self.assertEqual(session.cert, None)
        self.assertEqual(session.adapters['https://'].max_retries.total, 100)
        self.assertEqual(session.timeout, 99)

    def test_validate_connection(self):
        mock_conn = mocks.MockRequestSessionApi()
        mock_ep = mock.Mock()
        mock_ep.provider.url = 'https://1.2.3.4'
        provider = cluster.NSXRequestsHTTPProvider()
        self.assertRaises(nsx_exc.ResourceNotFound,
                          provider.validate_connection,
                          mock.Mock(), mock_ep, mock_conn)

        mock_conn.post('api/v1/transport-zones',
                       data=jsonutils.dumps({'id': 'dummy-tz'}),
                       headers=client.JSONRESTClient._DEFAULT_HEADERS)
        provider.validate_connection(mock.Mock(), mock_ep, mock_conn)


class NsxV3ClusteredAPITestCase(nsxlib_testcase.NsxClientTestCase):

    def _assert_providers(self, cluster_api, provider_tuples):
        self.assertEqual(len(cluster_api.providers), len(provider_tuples))

        def _assert_provider(pid, purl):
            for provider in cluster_api.providers:
                if provider.id == pid and provider.url == purl:
                    return
            self.fail("Provider: %s not found" % pid)

        for provider_tuple in provider_tuples:
            _assert_provider(provider_tuple[0], provider_tuple[1])

    def test_conf_providers_no_scheme(self):
        conf_managers = ['8.9.10.11', '9.10.11.12:4433']
        cfg.CONF.set_override(
            'nsx_api_managers', conf_managers, 'nsx_v3')

        mock_provider = mock.Mock()
        mock_provider.default_scheme = 'https'
        mock_provider.validate_connection = _validate_conn_up

        api = cluster.NSXClusteredAPI(http_provider=mock_provider)

        self._assert_providers(
            api, [(p, "https://%s" % p) for p in conf_managers])

    def test_conf_providers_with_scheme(self):
        conf_managers = ['http://8.9.10.11:8080', 'https://9.10.11.12:4433']
        cfg.CONF.set_override(
            'nsx_api_managers', conf_managers, 'nsx_v3')

        mock_provider = mock.Mock()
        mock_provider.default_scheme = 'https'
        mock_provider.validate_connection = _validate_conn_up

        api = cluster.NSXClusteredAPI(http_provider=mock_provider)

        self._assert_providers(
            api, [(urlparse.urlparse(p).netloc, p) for p in conf_managers])

    def test_http_retries(self):
        cfg.CONF.set_override(
            'http_retries', 9, 'nsx_v3')

        api = self.mock_nsx_clustered_api()
        with api.endpoints['1.2.3.4'].pool.item() as session:
            self.assertEqual(
                    session.adapters['https://'].max_retries.total, 9)

    def test_conns_per_pool(self):
        cfg.CONF.set_override(
            'concurrent_connections', 11, 'nsx_v3')
        conf_managers = ['8.9.10.11', '9.10.11.12:4433']
        cfg.CONF.set_override(
            'nsx_api_managers', conf_managers, 'nsx_v3')

        mock_provider = mock.Mock()
        mock_provider.default_scheme = 'https'
        mock_provider.validate_connection = _validate_conn_up

        api = cluster.NSXClusteredAPI(http_provider=mock_provider)

        for ep_id, ep in api.endpoints.items():
            self.assertEqual(ep.pool.max_size, 11)

    def test_timeouts(self):
        cfg.CONF.set_override(
            'http_read_timeout', 37, 'nsx_v3')
        cfg.CONF.set_override(
            'http_timeout', 7, 'nsx_v3')

        api = self.mock_nsx_clustered_api()
        api.get('logical-ports')
        mock_call = api.recorded_calls.method_calls[0]
        name, args, kwargs = mock_call
        self.assertEqual(kwargs['timeout'], (7, 37))


class ClusteredAPITestCase(nsxlib_testcase.NsxClientTestCase):

    def _test_health(self, validate_fn, expected_health):
        conf_managers = ['8.9.10.11', '9.10.11.12']
        cfg.CONF.set_override(
            'nsx_api_managers', conf_managers, 'nsx_v3')

        mock_provider = mock.Mock()
        mock_provider.default_scheme = 'https'

        mock_provider.validate_connection = validate_fn
        api = cluster.NSXClusteredAPI(http_provider=mock_provider)
        self.assertEqual(api.health, expected_health)

    def test_orange_health(self):

        def _validate(cluster_api, endpoint, conn):
            if endpoint.provider.id == '8.9.10.11':
                raise Exception()

        self._test_health(_validate, cluster.ClusterHealth.ORANGE)

    def test_green_health(self):
        self._test_health(_validate_conn_up, cluster.ClusterHealth.GREEN)

    def test_red_health(self):
        self._test_health(_validate_conn_down, cluster.ClusterHealth.RED)

    def test_cluster_validate_with_exception(self):
        conf_managers = ['8.9.10.11', '9.10.11.12', '10.11.12.13']
        cfg.CONF.set_override(
            'nsx_api_managers', conf_managers, 'nsx_v3')

        mock_provider = mock.Mock()
        mock_provider.default_scheme = 'https'

        mock_provider.validate_connection = _validate_conn_down
        api = cluster.NSXClusteredAPI(http_provider=mock_provider)

        self.assertEqual(len(api.endpoints), 3)
        self.assertRaises(nsx_exc.ServiceClusterUnavailable,
                          api.get, 'api/v1/transport-zones')

    def test_cluster_proxy_stale_revision(self):

        def stale_revision():
            raise nsx_exc.StaleRevision(manager='1.1.1.1',
                                        operation='whatever')

        api = self.mock_nsx_clustered_api(session_response=stale_revision)
        self.assertRaises(nsx_exc.StaleRevision,
                          api.get, 'api/v1/transport-zones')

    def test_cluster_proxy_connection_error(self):

        def connect_timeout():
            raise requests_exceptions.ConnectTimeout()

        api = self.mock_nsx_clustered_api(session_response=connect_timeout)
        api._validate = mock.Mock()
        self.assertRaises(nsx_exc.ServiceClusterUnavailable,
                          api.get, 'api/v1/transport-zones')

    def test_cluster_round_robin_servicing(self):
        conf_managers = ['8.9.10.11', '9.10.11.12', '10.11.12.13']
        cfg.CONF.set_override(
            'nsx_api_managers', conf_managers, 'nsx_v3')

        api = self.mock_nsx_clustered_api()
        api._validate = mock.Mock()

        eps = api._endpoints.values()

        def _get_schedule(num_eps):
            return [api._select_endpoint() for i in range(num_eps)]

        self.assertEqual(_get_schedule(3), eps)

        self.assertEqual(_get_schedule(6), [eps[0], eps[1], eps[2],
                                            eps[0], eps[1], eps[2]])

        eps[0]._state = cluster.EndpointState.DOWN
        self.assertEqual(_get_schedule(4), [eps[1], eps[2], eps[1], eps[2]])

        eps[1]._state = cluster.EndpointState.DOWN
        self.assertEqual(_get_schedule(2), [eps[2], eps[2]])

        eps[0]._state = cluster.EndpointState.UP
        self.assertEqual(_get_schedule(4), [eps[0], eps[2], eps[0], eps[2]])



# Copyright 2013 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import hashlib

from neutron.api.v2 import attributes
from neutron import version
from neutron_lib import exceptions
from oslo_config import cfg
from oslo_log import log
import retrying
import six

from vmware_nsx._i18n import _, _LE

LOG = log.getLogger(__name__)

MAX_DISPLAY_NAME_LEN = 40
MAX_RESOURCE_TYPE_LEN = 20
MAX_TAG_LEN = 40
NEUTRON_VERSION = version.version_info.release_string()
NSX_NEUTRON_PLUGIN = 'NSX Neutron plugin'
OS_NEUTRON_ID_SCOPE = 'os-neutron-id'


# Allowed network types for the NSX Plugin
class NetworkTypes:
    """Allowed provider network types for the NSX Plugin."""
    L3_EXT = 'l3_ext'
    STT = 'stt'
    GRE = 'gre'
    FLAT = 'flat'
    VLAN = 'vlan'
    BRIDGE = 'bridge'


# Allowed network types for the NSX-v Plugin
class NsxVNetworkTypes:
    """Allowed provider network types for the NSX-v Plugin."""
    FLAT = 'flat'
    VLAN = 'vlan'
    VXLAN = 'vxlan'
    PORTGROUP = 'portgroup'


# Allowed network types for the NSXv3 Plugin
class NsxV3NetworkTypes:
    """Allowed provider network types for the NSXv3 Plugin."""
    FLAT = 'flat'
    VLAN = 'vlan'
    VXLAN = 'vxlan'


def get_tags(**kwargs):
    tags = ([dict(tag=value, scope=key)
            for key, value in six.iteritems(kwargs)])
    tags.append({"tag": NEUTRON_VERSION, "scope": "quantum"})
    return sorted(tags, key=lambda x: x['tag'])


def device_id_to_vm_id(device_id, obfuscate=False):
    # device_id can be longer than 40 characters, for example
    # a device_id for a dhcp port is like the following:
    #
    # dhcp83b5fdeb-e3b4-5e18-ac5f-55161...80747326-47d7-46c2-a87a-cf6d5194877c
    #
    # To fit it into an NSX tag we need to hash it, however device_id
    # used for ports associated to VM's are small enough so let's skip the
    # hashing
    if len(device_id) > MAX_DISPLAY_NAME_LEN or obfuscate:
        return hashlib.sha1(device_id.encode()).hexdigest()
    else:
        return device_id or "N/A"


def check_and_truncate(display_name):
    if (attributes.is_attr_set(display_name) and
            len(display_name) > MAX_DISPLAY_NAME_LEN):
        LOG.debug("Specified name:'%s' exceeds maximum length. "
                  "It will be truncated on NSX", display_name)
        return display_name[:MAX_DISPLAY_NAME_LEN]
    return display_name or ''


def is_internal_resource(nsx_resource):
    """
    Indicates whether the passed nsx-resource is owned by the plugin for
    internal use.
    """
    for tag in nsx_resource.get('tags', []):
        if tag['scope'] == OS_NEUTRON_ID_SCOPE:
            return tag['tag'] == NSX_NEUTRON_PLUGIN
    return False


def build_v3_api_version_tag():
    """
    Some resources are created on the manager that do not have a corresponding
    Neutron resource.
    """
    return [{'scope': OS_NEUTRON_ID_SCOPE,
             'tag': NSX_NEUTRON_PLUGIN},
            {'scope': "os-api-version",
             'tag': version.version_info.release_string()}]


def _validate_resource_type_length(resource_type):
    # Add in a validation to ensure that we catch this at build time
    if len(resource_type) > MAX_RESOURCE_TYPE_LEN:
        raise exceptions.InvalidInput(
            error_message=(_('Resource type cannot exceed %(max_len)s '
                             'characters: %(resource_type)s') %
                           {'max_len': MAX_RESOURCE_TYPE_LEN,
                            'resource_type': resource_type}))


def build_v3_tags_payload(resource, resource_type, project_name):
    """
    Construct the tags payload that will be pushed to NSX-v3
    Add <resource_type>:<resource-id>, os-project-id:<tenant-id>,
    os-project-name:<project_name> os-api-version:<neutron-api-version>
    """
    _validate_resource_type_length(resource_type)
    # There may be cases when the plugin creates the port, for example DHCP
    if not project_name:
        project_name = 'NSX Neutron plugin'
    return [{'scope': resource_type,
             'tag': resource.get('id', '')[:MAX_TAG_LEN]},
            {'scope': 'os-project-id',
             'tag': resource.get('tenant_id', '')[:MAX_TAG_LEN]},
            {'scope': 'os-project-name',
             'tag': project_name[:MAX_TAG_LEN]},
            {'scope': 'os-api-version',
             'tag': version.version_info.release_string()[:MAX_TAG_LEN]}]


def add_v3_tag(tags, resource_type, tag):
    _validate_resource_type_length(resource_type)
    tags.append({'scope': resource_type, 'tag': tag[:MAX_TAG_LEN]})
    return tags


def update_v3_tags(tags, resources):
    port_tags = dict((t['scope'], t['tag']) for t in tags)
    resources = resources or []
    # Update tags
    for resource in resources:
        tag = resource['tag'][:MAX_TAG_LEN]
        resource_type = resource['resource_type']
        if resource_type in port_tags:
            if tag:
                port_tags[resource_type] = tag
            else:
                port_tags.pop(resource_type, None)
        else:
            port_tags[resource_type] = tag
    # Create the new set of tags
    return [{'scope': k, 'tag': v} for k, v in port_tags.items()]


def retry_upon_exception_nsxv3(exc, delay=500, max_delay=2000,
                               max_attempts=cfg.CONF.nsx_v3.retries):
    return retrying.retry(retry_on_exception=lambda e: isinstance(e, exc),
                          wait_exponential_multiplier=delay,
                          wait_exponential_max=max_delay,
                          stop_max_attempt_number=max_attempts)


def list_match(list1, list2):
    # Check if list1 and list2 have identical elements, but relaxed on
    # dict elements where list1's dict element can be a subset of list2's
    # corresponding element.
    if (not isinstance(list1, list) or
        not isinstance(list2, list) or
        len(list1) != len(list2)):
        return False
    list1 = sorted(list1)
    list2 = sorted(list2)
    for (v1, v2) in zip(list1, list2):
        if isinstance(v1, dict):
            if not dict_match(v1, v2):
                return False
        elif isinstance(v1, list):
            if not list_match(v1, v2):
                return False
        elif v1 != v2:
            return False
    return True


def dict_match(dict1, dict2):
    # Check if dict1 is a subset of dict2.
    if not isinstance(dict1, dict) or not isinstance(dict2, dict):
        return False
    for k1, v1 in dict1.items():
        if k1 not in dict2:
            return False
        v2 = dict2[k1]
        if isinstance(v1, dict):
            if not dict_match(v1, v2):
                return False
        elif isinstance(v1, list):
            if not list_match(v1, v2):
                return False
        elif v1 != v2:
            return False
    return True


def read_file(path):
    try:
        with open(path) as file:
            return file.read().strip()
    except IOError as e:
        LOG.error(_LE("Error while opening file "
                      "%(path)s: %(err)s"), {'path': path, 'err': str(e)})


def get_name_and_uuid(name, uuid, tag=None, maxlen=80):
    short_uuid = '_' + uuid[:5] + '...' + uuid[-5:]
    maxlen = maxlen - len(short_uuid)
    if tag:
        maxlen = maxlen - len(tag) - 1
        return name[:maxlen] + '_' + tag + short_uuid
    else:
        return name[:maxlen] + short_uuid


def is_ipv4_ip_address(addr):

    def _valid_part(part):
        try:
            int_part = int(part)
            if int_part < 0 or int_part > 255:
                return False
            return True
        except ValueError:
            return False

    parts = str(addr).split('.')
    if len(parts) != 4:
        return False

    for ip_part in parts:
        if not _valid_part(ip_part):
            return False
    return True



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.common.utils import data_utils
from tempest import config
from tempest import test

from vmware_nsx_tempest.services import nsxv3_client

CONF = config.CONF


class NSXv3NetworksTest(base.BaseNetworkTest):
    """Tests the following operations in the Neutron API:
       - Create network
       - Update network
       - Delete network
    After the neutron API call, we also need to make sure the corresponding
    resource has been created/updated/deleted from NSX backend.
    """

    @classmethod
    def resource_setup(cls):
        super(NSXv3NetworksTest, cls).resource_setup()
        cls.nsx = nsxv3_client.NSXV3Client(CONF.nsxv3.nsx_manager,
                                           CONF.nsxv3.nsx_user,
                                           CONF.nsxv3.nsx_password)

    @test.attr(type='nsxv3')
    @test.idempotent_id('63085723-23ae-4109-ac86-69f895097957')
    def test_create_update_delete_nsx_network(self):
        # Create a network
        name = data_utils.rand_name('network-')
        network = self.create_network(network_name=name)
        net_id = network['id']
        nsx_network = self.nsx.get_logical_switch(network['name'],
                                                  network['id'])
        self.assertEqual('ACTIVE', network['status'])
        self.assertNotEqual(nsx_network, None)
        # Verify network update
        new_name = "New_network"
        body = self.networks_client.update_network(net_id, name=new_name)
        updated_net = body['network']
        nsx_network = self.nsx.get_logical_switch(updated_net['name'],
                                                  updated_net['id'])
        self.assertEqual(updated_net['name'], new_name)
        self.assertNotEqual(nsx_network, None)
        # Verify delete network
        self.networks_client.delete_network(updated_net['id'])
        nsx_network = self.nsx.get_logical_switch(updated_net['name'],
                                                  updated_net['id'])
        self.assertEqual(nsx_network, None)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


# netowrk/json/base.py does not include thoese method in network_client
class BaseNetworkClient(base.BaseNetworkClient):
    def __init__(self, auth_provider, service, region,
                 endpoint_type=None, build_interval=None, build_timeout=None,
                 disable_ssl_certificate_validation=None, ca_certs=None,
                 trace_requests=None, **kwargs):
        dsca = disable_ssl_certificate_validation
        super(base.BaseNetworkClient, self).__init__(
            auth_provider, service, region,
            endpoint_type=endpoint_type,
            build_interval=build_interval,
            build_timeout=build_timeout,
            disable_ssl_certificate_validation=dsca,
            ca_certs=ca_certs,
            trace_requests=trace_requests)


default_params = {
    'disable_ssl_certificate_validation': True,
    'ca_certs': None,
    'trace_requests': ''}
default_params_2 = {
    'catalog_type': 'network',
    'region': 'nova',
    'endpoint_type': 'publicURL',
    'build_timeout': 300,
    'build_interval': 1}
